from quantulum3 import parser
from quantulum3.load import add_custom_unit
import re

def clean_query(query_raw):
    
    # add space between numbers and charactoers
    query_processed = re.sub(r"([0-9]+(\.[0-9]+)?)",r" \1 ", query_raw).strip()
    
    # remove extra spaces other than 1 space
    query_processed = " ".join(query_processed.split())
    
    return query_processed

def unit_convertor(unit_entity, val, unit_in, unit_out):
    # check unit_entity is from ["length", "mass", "volume"]
    # and perform corresponding conversion
    if unit_entity == "length":
        coversion_dic = {'mm':0.001, 'cm':0.01, 'm':1.0, 'km':1000.}
    elif unit_entity == "mass":
        coversion_dic = {'oz':28, 'lb':454, 'g':1.0, 'kg':1000.}
    else:
        coversion_dic = {'ml':0.001, 'l':1.0}
        
    return val*coversion_dic[unit_in]/coversion_dic[unit_out]

def unit_standardization(numbers_quantifier, standard_length, standard_mass, standard_volume):
    # get info 
    unit = numbers_quantifier["unit"]
    quantity = numbers_quantifier["quantity"]
    unit_entity = numbers_quantifier["unit_entity"]
    
    # create mapping
    valid_standard_quantity_pool = ["mass", "volume", "length"]
    unit_entity_mapping = {"mass":standard_mass, "volume":standard_volume, "length":standard_length}
    
    # if unit_entity can be standardized, we perform the conversion
    if unit_entity in valid_standard_quantity_pool:
        numbers_quantifier["standardization"] = True
        numbers_quantifier["unit_standardized"] = unit_entity_mapping[unit_entity]
        numbers_quantifier["quantity_standardized"] = unit_convertor(unit_entity, quantity, unit, unit_entity_mapping[unit_entity])
        
    else:
        numbers_quantifier["standardization"] = False
    
    return numbers_quantifier

def numbers_quantifier(query_raw, standard_length, standard_mass, standard_volume):
    
    # clean query
    query_rewrite = clean_query(query_raw)
    print(query_rewrite)
    # initiate 
    numbers_quantifier = {}
    parse_status = False
    # start parsing
    try:
        # parse and grab information
        quants = parser.parse(query_rewrite)
        tokens = quants[0].surface
        quantity = quants[0].value
        quantity_span = quants[0].span
        unit_name = quants[0].unit.name
        unit_entity = quants[0].unit.entity.name
        
        # update parse status
        parse_status = True
        
        # update result
        numbers_quantifier["query_raw"] = query_raw
        numbers_quantifier["query_rewrite"] = query_rewrite
        numbers_quantifier["parse_status"] = parse_status
        numbers_quantifier["tokens"] = tokens
        numbers_quantifier["quantity"] = quantity
        numbers_quantifier["quantity_span"] = quantity_span
        numbers_quantifier["unit"] = tokens.split()[1].lower()
        numbers_quantifier["unit_name"] = unit_name
        if "size" in query_rewrite:
            numbers_quantifier["unit_entity"] = "size"
        else:
            numbers_quantifier["unit_entity"] = unit_entity
        
    except:
        numbers_quantifier["parse_status"] = parse_status
        
    # edit custom units
    add_custom_unit(name="bottle", surfaces=["bottle"], entity="dimensionless")
    add_custom_unit(name="size", surfaces=["size"], entity="dimensionless")
    add_custom_unit(name="serving", surfaces=["serving"], entity="dimensionless")
    add_custom_unit(name="roll", surfaces=["roll"], entity="dimensionless")
    add_custom_unit(name="pack", surfaces=["pack"], entity="dimensionless")
    add_custom_unit(name="each", surfaces=["ea"], entity="dimensionless")
    
    # unit standardization
    numbers_quantifier = unit_standardization(numbers_quantifier, standard_length, standard_mass, standard_volume)
    
    return numbers_quantifier